#!/usr/bin/perl
#
# Jonathan Klassen
# May 23, 2012
#
# test_parser.pl

use warnings;
use Math::NumberCruncher;

open (INFILE, $ARGV[0]) or die "The test sequence set is not ARGV[0]";
while (<INFILE>){
	chomp;
	if (/^>/){
		$seqs++;
	}
	next unless (/^>/ and /\|/);
	@line = split /\|/;
	$line[0] =~ /^>(\w+) / or die;
	$seq_name = $1;
	shift @line;
	foreach (@line){
		/(\w+) ((\d|,)+)/ or die;
		$ref_errors{$seq_name}{$1} = $2;
	}
}
close INFILE;

$five_short_tp = $five_short_fp = $five_short_fn = $five_short_perfect = $five_long_tp = $five_long_fp = $five_long_fn = $five_long_perfect = $five_chimera_tp = $five_chimera_fp = $five_chimera_fn = $five_chimera_perfect = $three_short_tp = $three_short_fp = $three_short_fn = $three_short_perfect = $three_long_tp = $three_long_fp = $three_long_fn = $three_long_perfect = $three_chimera_tp = $three_chimera_fp = $three_chimera_fn = $three_chimera_perfect = $five_short_fn = $five_long_fn = $five_chimera_fn = $three_short_fn = $three_long_fn = $three_chimera_fn = 0;
@five_short_mismatches = @five_short_average = @five_short_fp = @five_long_mismatches = @five_long_average = @five_long_fp = @five_chimera_mismatches = @five_chimera_average = @five_chimera_fp = @three_short_mismatches = @three_short_average = @three_short_fp = @three_long_mismatches = @three_long_average = @three_long_fp = @three_chimera_mismatches = @three_chimera_average = @three_chimera_fp = @five_short_fn = @five_long_fn = @five_chimera_fn = @three_short_fn = @three_long_fn = @three_chimera_fn = ();

open (INFILE, $ARGV[1]) or die "The corrected sequence details output file is not ARGV[1]";
while (<INFILE>){
	chomp;
	if (/Sequence having short 5' ends: (\w+)\t(\d+) amino acid/){
		$seq_name = $1;
		$corrected_aas = $2;
		if ($ref_errors{$seq_name}{'five_short'}){
			$five_short_tp++;
			if ($corrected_aas == $ref_errors{$seq_name}{'five_short'}){
				$five_short_perfect++;
			}
			else {
				push @five_short_mismatches, abs($corrected_aas - $ref_errors{$seq_name}{'five_short'});
			}
			push @five_short_average, abs($corrected_aas - $ref_errors{$seq_name}{'five_short'});
		}
		else {
			$five_short_fp++;
			push @five_short_fp, $corrected_aas;
		}
		delete $ref_errors{$seq_name}{'five_short'};
	}
	elsif (/Sequence having long 5' ends: (\w+)\t(\d+) amino acid/){
		$seq_name = $1;
		$corrected_aas = $2;
		if ($ref_errors{$seq_name}{'five_long'}){
			$five_long_tp++;
			if ($corrected_aas == $ref_errors{$seq_name}{'five_long'}){
				$five_long_perfect++;
			}
			else {
				push @five_long_mismatches, abs($corrected_aas - $ref_errors{$seq_name}{'five_long'});
			}
			push @five_long_average, abs($corrected_aas - $ref_errors{$seq_name}{'five_long'});
		}
		else {
			$five_long_fp++;
			push @five_long_fp, $corrected_aas;
		}
		delete $ref_errors{$seq_name}{'five_long'};
	}
	elsif (/Sequence having 5' chimeras: (\w+)\t(\d+) amino acid/){
		$seq_name = $1;
		$corrected_aas = $2;
		if ($ref_errors{$seq_name}{'five_chimera'}){
			$five_chimera_tp++;
			@temp = ();
			@temp = split ",", $ref_errors{$seq_name}{'five_chimera'};
			if ($corrected_aas == $temp[1]){
				$five_chimera_perfect++;
			}
			else {
				push @five_chimera_mismatches, abs($corrected_aas - $temp[1]);
			}
			push @five_chimera_average, abs($corrected_aas - $temp[1]);
		}
		else {
			$five_chimera_fp++;
			push @five_chimera_fp, $corrected_aas;
		}
		delete $ref_errors{$seq_name}{'five_chimera'};
	}
	elsif (/Sequence having short 3' ends: (\w+)\t(\d+) amino acid/){
		$seq_name = $1;
		$corrected_aas = $2;
		if ($ref_errors{$seq_name}{'three_short'}){
			$three_short_tp++;
			if ($corrected_aas == $ref_errors{$seq_name}{'three_short'}){
				$three_short_perfect++;
			}
			else {
				push @three_short_mismatches, abs($corrected_aas - $ref_errors{$seq_name}{'three_short'});
			}
			push @three_short_average, abs($corrected_aas - $ref_errors{$seq_name}{'three_short'});
		}
		else {
			$three_short_fp++;
			push @three_short_fp, $corrected_aas;
		}
		delete $ref_errors{$seq_name}{'three_short'};
	}
	elsif (/Sequence having long 3' ends: (\w+)\t(\d+) amino acid/){
		$seq_name = $1;
		$corrected_aas = $2;
		if ($ref_errors{$seq_name}{'three_long'}){
			$three_long_tp++;
			if ($corrected_aas == $ref_errors{$seq_name}{'three_long'}){
				$three_long_perfect++;
			}
			else {
				push @three_long_mismatches, abs($corrected_aas - $ref_errors{$seq_name}{'three_long'});
			}
			push @three_long_average, abs($corrected_aas - $ref_errors{$seq_name}{'three_long'});
		}
		else {
			$three_long_fp++;
			push @three_long_fp, $corrected_aas;
		}
		delete $ref_errors{$seq_name}{'three_long'};
	}
	elsif (/Sequence having 3' chimeras: (\w+)\t(\d+) amino acid/){
		$seq_name = $1;
		$corrected_aas = $2;
		if ($ref_errors{$seq_name}{'three_chimera'}){
			$three_chimera_tp++;
			@temp = ();
			@temp = split ",", $ref_errors{$seq_name}{'three_chimera'};
			if ($corrected_aas == $temp[1]){
				$three_chimera_perfect++;
			}
			else {
				push @three_chimera_mismatches, abs($corrected_aas - $temp[1]);
			}
			push @three_chimera_average, abs($corrected_aas - $temp[1]);
		}
		else {
			$three_chimera_fp++;
			push @three_chimera_fp, $corrected_aas;
		}
		delete $ref_errors{$seq_name}{'three_chimera'};
	}
}
foreach $seq_name (sort keys %ref_errors){
	if ($ref_errors{$seq_name}{'five_short'}){
		$five_short_fn++;
		push @five_short_fn, $ref_errors{$seq_name}{'five_short'};
		delete $ref_errors{$seq_name}{'five_short'};
	}
	elsif ($ref_errors{$seq_name}{'five_long'}){
		$five_long_fn++;
		push @five_long_fn, $ref_errors{$seq_name}{'five_long'};
		delete $ref_errors{$seq_name}{'five_long'};
	}
	elsif ($ref_errors{$seq_name}{'five_chimera'}){
		$five_chimera_fn++;
		@temp = ();
		@temp = split ",", $ref_errors{$seq_name}{'five_chimera'};
		push @five_chimera_fn, $temp[1];
		delete $ref_errors{$seq_name}{'five_chimera'};
	}
	elsif ($ref_errors{$seq_name}{'three_short'}){
		$three_short_fn++;
		push @three_short_fn, $ref_errors{$seq_name}{'three_short'};
		delete $ref_errors{$seq_name}{'three_short'};
	}
	elsif ($ref_errors{$seq_name}{'three_long'}){
		$three_long_fn++;
		push @three_long_fn, $ref_errors{$seq_name}{'three_long'};
		delete $ref_errors{$seq_name}{'three_long'};
	}
	elsif ($ref_errors{$seq_name}{'three_chimera'}){
		$three_chimera_fn++;
		@temp = ();
		@temp = split ",", $ref_errors{$seq_name}{'three_chimera'};
		push @three_chimera_fn, $temp[1];
		delete $ref_errors{$seq_name}{'three_chimera'};
	}
}
$five_short_tn = $seqs - $five_short_tp - $five_short_fp - $five_short_fn;
$five_long_tn = $seqs - $five_long_tp - $five_long_fp - $five_long_fn;
$five_chimera_tn = $seqs - $five_chimera_tp - $five_chimera_fp - $five_chimera_fn;
$three_short_tn = $seqs - $three_short_tp - $three_short_fp - $three_short_fn;
$three_long_tn = $seqs - $three_long_tp - $three_long_fp - $three_long_fn;
$three_chimera_tn = $seqs - $three_chimera_tp - $three_chimera_fp - $three_chimera_fn;

$five_short_sensitivity = ($five_short_tp / ($five_short_tp + $five_short_fn)) * 100;
$five_short_specificity = ($five_short_tn / ($five_short_tn + $five_short_fp)) * 100;
$five_long_sensitivity = ($five_long_tp / ($five_long_tp + $five_long_fn)) * 100;
$five_long_specificity = ($five_long_tn / ($five_long_tn + $five_long_fp)) * 100;
$five_chimera_sensitivity = ($five_chimera_tp / ($five_chimera_tp + $five_chimera_fn)) * 100;
$five_chimera_specificity = ($five_chimera_tn / ($five_chimera_tn + $five_chimera_fp)) * 100;
$three_short_sensitivity = ($three_short_tp / ($three_short_tp + $three_short_fn)) * 100;
$three_short_specificity = ($three_short_tn / ($three_short_tn + $three_short_fp)) * 100;
$three_long_sensitivity = ($three_long_tp / ($three_long_tp + $three_long_fn)) * 100;
$three_long_specificity = ($three_long_tn / ($three_long_tn + $three_long_fp)) * 100;
$three_chimera_sensitivity = ($three_chimera_tp / ($three_chimera_tp + $three_chimera_fn)) * 100;
$three_chimera_specificity = ($three_chimera_tn / ($three_chimera_tn + $three_chimera_fp)) * 100;

open (OUTFILE, ">$ARGV[2]") or die "Cannot open outfile ARGV[2]";
print OUTFILE 
	"five_short_tp\tfive_short_fp\tfive_short_tn\tfive_short_fn\t",
	"five_long_tp\tfive_long_fp\tfive_long_tn\tfive_long_fn\t",
	"five_chimera_tp\tfive_chimera_fp\tfive_chimera_tn\tfive_chimera_fn\t",
	"three_short_tp\tthree_short_fp\tthree_short_tn\tthree_short_fn\t",
	"three_long_tp\tthree_long_fp\tthree_long_tn\tthree_long_fn\t",
	"three_chimera_tp\tthree_chimera_fp\tthree_chimera_tn\tthree_chimera_fn\t",
	"five_short_perfect\tfive_long_perfect\tfive_chimera_perfect\tthree_short_perfect\tthree_long_perfect\tthree_chimera_perfect\t",
	"five_short_sensitivity\tfive_short_specificity\tfive_long_sensitivity\tfive_long_specificity\tfive_chimera_sensitivity\tfive_chimera_specificity\t",
	"three_short_sensitivity\tthree_short_specificity\tthree_long_sensitivity\tthree_long_specificity\tthree_chimera_sensitivity\tthree_chimera_specificity\t",
	"five_short_mismatches\tfive_short_accuracy\tfive_short_fn_lengths\t",
	"five_long_mismatches\tfive_long_accuracy\tfive_long_fn_lengths\t",
	"five_chimera_mismatches\tthree_chimera_accuracy\tfive_chimera_fn_lengths\t",
	"three_short_mismatches\tthree_short_accuracy\tthree_short_fn_lengths\t",
	"three_long_mismatches\tthree_long_accuracy\tthree_long_fn_lengths\t",
	"three_chimera_mismatches\tthree_chimera_accuracy\tthree_chimera_fn_lengths\n";
print OUTFILE 
	"$five_short_tp\t$five_short_fp\t$five_short_tn\t$five_short_fn\t",
	"$five_long_tp\t$five_long_fp\t$five_long_tn\t$five_long_fn\t",
	"$five_chimera_tp\t$five_chimera_fp\t$five_chimera_tn\t$five_chimera_fn\t",
	"$three_short_tp\t$three_short_fp\t$three_short_tn\t$three_short_fn\t",
	"$three_long_tp\t$three_long_fp\t$three_long_tn\t$three_long_fn\t",
	"$three_chimera_tp\t$three_chimera_fp\t$three_chimera_tn\t$three_chimera_fn\t",
	"$five_short_perfect\t$five_long_perfect\t$five_chimera_perfect\t$three_short_perfect\t$three_long_perfect\t$three_chimera_perfect\t", 
	"$five_short_sensitivity\t$five_short_specificity\t$five_long_sensitivity\t$five_long_specificity\t$five_chimera_sensitivity\t$five_chimera_specificity\t",
	"$three_short_sensitivity\t$three_short_specificity\t$three_long_sensitivity\t$three_long_specificity\t$three_chimera_sensitivity\t$three_chimera_specificity\t",
	"@five_short_mismatches\t@five_short_average\t@five_short_fn\t",
	"@five_long_mismatches\t@five_long_average\t@five_long_fn\t",
	"@five_chimera_mismatches\t@five_chimera_average\t@five_chimera_fn\t",
	"@three_short_mismatches\t@three_short_average\t@three_short_fn\t",
	"@three_long_mismatches\t@three_long_average\t@three_long_fn\t",
	"@three_chimera_mismatches\t@three_chimera_average\t@three_chimera_fn\n";


