#!/usr/bin/perl
#
# Jonathan Klassen
# May 18, 2012
#
# validation_dataset_maker_v2.pl
# v2: uses common input sequence file, common "contaminant" input
# v3: excludes sequences without 5 matches from being altered
# v4: does not use blast to exclude references (assumed to be perfect)

# loads modified seqs

use Math::Random::MT::Auto qw(poisson);

$five_chimera_rate = $ARGV[0];
$five_long_rate = $ARGV[1];
$five_short_rate = $ARGV[2];
$three_chimera_rate = $ARGV[3];
$three_long_rate = $ARGV[4];
$three_short_rate = $ARGV[5];
$five_chimera_offset = $ARGV[6];
$five_add_delete_offset = $ARGV[7];
$three_chimera_offset = $ARGV[8];
$three_add_delete_offset = $ARGV[9];
$id = $ARGV[10] or die;

# harvests input seqs

open (INFILE, "input_seqs.faa") or die;
while (<INFILE>){
	chomp;
	if (/^>/){
		$header = $_;
		/(>\w+ (([A-Z]|[a-z])+)\w+ (\w+))/ or die "Cannot match";
		$genus_lookup{$1} = $2;
		$cog_lookup{$1} = $4;
		$five_genus_count{$4}{$2}++;
		$three_genus_count{$4}{$2}++;
	}
	else {
		die if (!$header);
		$outseqs{$header} = $_;
	}
}

# harvests sequences to turn into contaminants

open (INFILE, "contaminant_seqs.faa") or die;
while (<INFILE>){
	chomp;
	next if (/>/);
	if ($seq){
		$seq = $seq.$_;
	}
	else {
		$seq = $_;
	}
}
$seq =~ tr/[A-Z]/[a-z]/;

# performs all vs all BLAST of the input seqs

#system ("makeblastdb -in input_seqs.faa");
#system ("blastp -db input_seqs.faa -evalue 1e-10 -query input_seqs.faa -num_threads 8 -out input_seqs_vs_self.blastp -max_target_seqs 6 -outfmt \"7 qseqid sseqid qlen slen qstart qend sstart send length evalue bitscore nident positive\"");

#open (INBLAST, "input_seqs_vs_self.blastp") or die;
#while (<INBLAST>){
#	if (/^\# Query: (.+)$/){
#print "$query $number_of_hits\n";
#		$query = $1;
#		$number_of_hits = 0;
#		$skip{">".$query} = "y";
#	}
#	elsif (/^\# (\d+) hits found$/){
#		if ($1 == 6){
#			delete $skip{">".$query};
#		}
#	}
#	elsif (/^\#/){
#		next;
#	}
#	else {
#		@line = split "\t";
#		unless ($line[11] / $line[8] < $id){ 
#			$number_of_hits++; 
#		}
#		if ($number_of_hits == 6){
#			delete $skip{">".$query};
#		}
#	}
#}
#print sort keys %skip;
#print "selecting sequences to alter...\n";

# randomly select sequences to alter 
# forces there to be at least 5 seqs in each genus (such that correction is actually possible)

$counter = -1;
foreach (keys %outseqs){
	push @outseq_array, $_;
	$counter++;
	$lookup{$_} = $_;
}

while (!%found_five_chimera or scalar keys %found_five_chimera < ($five_chimera_rate * scalar keys %outseqs)){	
	$rand = int(rand $#outseq_array);
	next if ($skip{$outseq_array[$rand]} eq "y");
	next if ($found_five_chimera{$outseq_array[$rand]} eq "y");
	next if ($five_genus_count{$cog_lookup{$outseq_array[$rand]}}{$genus_lookup{$outseq_array[$rand]}} <= 5);
	$five_genus_count{$cog_lookup{$outseq_array[$rand]}}{$genus_lookup{$outseq_array[$rand]}}--;
	$found_five_chimera{$outseq_array[$rand]} = "y";
}
while (!%found_five_long or scalar keys %found_five_long < ($five_long_rate * scalar keys %outseqs)){
	$rand = int(rand scalar keys %outseqs);
	next if ($skip{$outseq_array[$rand]} eq "y");
	next if ($found_five_chimera{$outseq_array[$rand]} eq "y");
	next if ($found_five_long{$outseq_array[$rand]} eq "y");
	next if ($five_genus_count{$cog_lookup{$outseq_array[$rand]}}{$genus_lookup{$outseq_array[$rand]}} <= 5);
	$five_genus_count{$cog_lookup{$outseq_array[$rand]}}{$genus_lookup{$outseq_array[$rand]}}--;
	$found_five_long{$outseq_array[$rand]} = "y";
}
while (!%found_five_short or scalar keys %found_five_short < ($five_short_rate * scalar keys %outseqs)){ 
	$rand = int(rand scalar keys %outseqs);
	next if ($skip{$outseq_array[$rand]} eq "y");
	next if ($found_five_chimera{$outseq_array[$rand]} eq "y");
	next if ($found_five_long{$outseq_array[$rand]} eq "y");
	next if ($found_five_short{$outseq_array[$rand]} eq "y");
	next if ($five_genus_count{$cog_lookup{$outseq_array[$rand]}}{$genus_lookup{$outseq_array[$rand]}} <= 5);
	$five_genus_count{$cog_lookup{$outseq_array[$rand]}}{$genus_lookup{$outseq_array[$rand]}}--;
	$found_five_short{$outseq_array[$rand]} = "y";
}
while (!%found_three_chimera or scalar keys %found_three_chimera < ($three_chimera_rate * scalar keys %outseqs)){ 
	$rand = int(rand scalar keys %outseqs);
	next if ($skip{$outseq_array[$rand]} eq "y");
	next if ($found_three_chimera{$outseq_array[$rand]} eq "y");
	next if ($found_five_chimera{$outseq_array[$rand]} eq "y");
	next if ($found_five_long{$outseq_array[$rand]} eq "y");
	next if ($found_five_short{$outseq_array[$rand]} eq "y");
	next if ($three_genus_count{$cog_lookup{$outseq_array[$rand]}}{$genus_lookup{$outseq_array[$rand]}} <= 5);
	$three_genus_count{$cog_lookup{$outseq_array[$rand]}}{$genus_lookup{$outseq_array[$rand]}}--;
	$found_three_chimera{$outseq_array[$rand]} = "y";
}
while (!%found_three_long or scalar keys %found_three_long < ($three_long_rate * scalar keys %outseqs)){
	$rand = int(rand scalar keys %outseqs);
	$here = "y";
	next if ($skip{$outseq_array[$rand]} eq "y");
	next if ($found_three_chimera{$outseq_array[$rand]} eq "y");
	next if ($found_three_long{$outseq_array[$rand]} eq "y");
	next if ($found_five_chimera{$outseq_array[$rand]} eq "y");
	next if ($three_genus_count{$cog_lookup{$outseq_array[$rand]}}{$genus_lookup{$outseq_array[$rand]}} <= 5);
	$three_genus_count{$cog_lookup{$outseq_array[$rand]}}{$genus_lookup{$outseq_array[$rand]}}--;
	$found_three_long{$outseq_array[$rand]} = "y";
}
while (!%found_three_short or scalar keys %found_three_short < ($three_short_rate * scalar keys %outseqs)){ 
	$rand = int(rand scalar keys %outseqs);
	next if ($skip{$outseq_array[$rand]} eq "y");
	next if ($found_three_chimera{$outseq_array[$rand]} eq "y");
	next if ($found_three_long{$outseq_array[$rand]} eq "y");
	next if ($found_three_short{$outseq_array[$rand]} eq "y");
	next if ($found_five_chimera{$outseq_array[$rand]} eq "y");
	next if ($three_genus_count{$cog_lookup{$outseq_array[$rand]}}{$genus_lookup{$outseq_array[$rand]}} <= 5);
	$three_genus_count{$cog_lookup{$outseq_array[$rand]}}{$genus_lookup{$outseq_array[$rand]}}--;
	$found_three_short{$outseq_array[$rand]} = "y";
}

#print "altering sequences...\n";

# select sequence length to alter

#print "\t5 chimera\n";

foreach $key (keys %found_five_chimera){
	$poisson_delete = abs(poisson(1)) + $five_chimera_offset;
	$poisson_add = abs(poisson(1)) + $five_chimera_offset;
	$go = "n";
	while ($go eq "n"){
		$rand = int(rand(length $seq)) - 1;
		next if ($rand + $poisson > length $seq);
		$go = "y";
	}
	$subseq = substr($seq, $rand, $poisson_add);
	$new_key = $lookup{$key};
	$new_key = $new_key."|five_chimera $poisson_delete,$poisson_add";
	substr($outseqs{$lookup{$key}}, 0, $poisson_delete, $subseq);
	$outseqs{$new_key} = $outseqs{$lookup{$key}};
	$replaced_lookup{$lookup{$key}} = $new_key;
	delete $outseqs{$lookup{$key}};
}

#print "\t5 long\n";

foreach $key (keys %found_five_long){
	$poisson = abs(poisson(1)) + $five_add_delete_offset;
	$go = "n";
	while ($go eq "n"){
		$rand = int(rand(length $seq)) - 1;
		next if ($rand + $poisson > length $seq);
		$go = "y";
	}
	$subseq = substr($seq, $rand, $poisson);
	$new_key = $lookup{$key};
	$new_key = $new_key."|five_long $poisson";
	$outseqs{$new_key} = $subseq.$outseqs{$lookup{$key}};
	$replaced_lookup{$lookup{$key}} = $new_key;
	delete $outseqs{$lookup{$key}};
}

#print "\t5 short\n";

foreach $key (keys %found_five_short){
	$poisson = abs(poisson(1)) + $five_add_delete_offset;
	$go = "n";
	while ($go eq "n"){
		$rand = int(rand(length $seq)) - 1;
		next if ($rand + $poisson > length $seq);
		$go = "y";
	}
	$subseq = substr($seq, $rand, $poisson);
	$new_key = $lookup{$key};
	$new_key = $new_key."|five_short $poisson";
	$outseqs{$new_key} = substr($outseqs{$lookup{$key}}, $poisson, (length $outseqs{$lookup{$key}}) - $poisson);
	$replaced_lookup{$lookup{$key}} = $new_key;
	delete $outseqs{$lookup{$key}};
}

#print "\t3 chimera\n";

foreach $key (keys %found_three_chimera){
	$poisson_delete = abs(poisson(1)) + $three_chimera_offset;
	$poisson_add = abs(poisson(1)) + $three_chimera_offset;
	$go = "n";
	while ($go eq "n"){
		$rand = int(rand(length $seq)) - 1;
		next if ($rand + $poisson > length $seq);
		$go = "y";
	}
	$subseq = substr($seq, $rand, $poisson_add);
	if ($outseqs{$lookup{$key}}){
		$new_key = $lookup{$key};
		$new_key = $new_key."|three_chimera $poisson_delete,$poisson_add";
		substr($outseqs{$lookup{$key}}, (length $outseqs{$lookup{$key}}) - 1 - $poisson_delete, (length $outseqs{$lookup{$key}}), $subseq);
		$outseqs{$new_key} = $outseqs{$lookup{$key}};
		delete $outseqs{$lookup{$key}};
	}
	else {
		$new_key = $replaced_lookup{$lookup{$key}};die "$key * $lookup{$key} * ", scalar keys %lookup, " ", scalar keys %outseqs;
		$new_key = $new_key."|three_chimera $poisson_delete,$poisson_add";
		substr($outseqs{$replaced_lookup{$lookup{$key}}}, (length $outseqs{$replaced_lookup{$lookup{$key}}}) - $poisson_delete, (length $outseqs{$replaced_lookup{$lookup{$key}}}) - 1, $subseq);
		$outseqs{$new_key} = $outseqs{$replaced_lookup{$lookup{$key}}};
		delete $outseqs{$replaced_lookup{$lookup{$key}}};
	}		
}

#print "\t3 long\n";

foreach $key (keys %found_three_long){
	$poisson = abs(poisson(1)) + $three_add_delete_offset;
	$go = "n";
	while ($go eq "n"){
		$rand = int(rand(length $seq)) - 1;
		next if ($rand + $poisson > length $seq);
		$go = "y";
	}
	$subseq = substr($seq, $rand, $poisson);
	if ($outseqs{$lookup{$key}}){
		$new_key = $lookup{$key};
		$new_key = $new_key."|three_long $poisson";
		$outseqs{$new_key} = $outseqs{$lookup{$key}}.$subseq;
		delete $outseqs{$lookup{$key}};
	}
	else {
		$new_key = $replaced_lookup{$lookup{$key}};
		$new_key = $new_key."|three_long $poisson";
		$outseqs{$new_key} = $outseqs{$replaced_lookup{$lookup{$key}}}.$subseq;
		delete $outseqs{$replaced_lookup{$lookup{$key}}};
	}
}

#print "\t3 short\n";

foreach $key (keys %found_three_short){
	$poisson = abs(poisson(1)) + $three_add_delete_offset;
	$go = "n";
	while ($go eq "n"){
		$rand = int(rand(length $seq)) - 1;
		next if ($rand + $poisson > length $seq);
		$go = "y";
	}
	$subseq = substr($seq, $rand, $poisson);
	if ($outseqs{$lookup{$key}}){
		$new_key = $lookup{$key};
		$new_key = $new_key."|three_short $poisson";
		$outseqs{$new_key} = substr($outseqs{$lookup{$key}}, 0, (length $outseqs{$lookup{$key}}) - $poisson);
		delete $outseqs{$lookup{$key}};
	}
	else {
		$new_key = $replaced_lookup{$lookup{$key}};
		$new_key = $new_key."|three_short $poisson";
		$outseqs{$new_key} = substr($outseqs{$replaced_lookup{$lookup{$key}}}, 0, (length $outseqs{$replaced_lookup{$lookup{$key}}}) - $poisson);
		delete $outseqs{$replaced_lookup{$lookup{$key}}};	
	}
}

# output new modified sequences

open (OUTFILE, ">altered_seqs.faa") or die;
foreach $keep (sort keys %outseqs){
	next if ($skip{$keep});
	next unless ($keep =~ />\w+ \w+ (\w+)/);
	open (EACHOUTFILE, ">>$1.faa") or die;
	print EACHOUTFILE "$keep\n$outseqs{$keep}\n";
	close EACHOUTFILE;
	print OUTFILE "$keep\n$outseqs{$keep}\n";
}
close OUTFILE;
